@@ -12,7 +12,11 @@ from distutils.spawn import find_executable
 matam_16s_usage = '''
 =================================== matam_16s example commands ===================================
 
-MarkerMAG matam_16s -p Test -r1 R1.fasta -r2 R2.fasta -t 12 -force -d path/to/SILVA_138_SSURef_NR99
+MarkerMAG matam_16s -p Test -r1 R1.fasta -r2 R2.fasta -t 12 -d path/to/SILVA_138_SSURef_NR99
+MarkerMAG matam_16s -p Test -r16s 16S_reads.fasta -t 12 -d path/to/SILVA_138_SSURef_NR99
+
+# please refer to here for the preparation of input files
+https://github.com/songweizhi/MarkerMAG/blob/master/doc/README_matam_16s.md
 
 ==================================================================================================
 '''
@@ -65,9 +69,19 @@ def str_to_num_list(nums_str):
     return sorted(subsample_pct_list)
 
 
-def sep_paired_and_singleton_reads(fasta_in, fasta_out_r1, fasta_out_r2, fasta_out_singleton):
+def sep_paired_and_singleton_reads(seq_in, seq_out_r1, seq_out_r2, seq_out_singleton):
+
+    input_seq_ext = os.path.splitext(seq_in)[1]
+    input_seq_fmt = 'fasta'
+    if ('q' in input_seq_ext) or ('Q' in input_seq_ext):
+        input_seq_fmt = 'fastq'
+
+    fmt_to_provide_for_write = 'fasta-2line'
+    if input_seq_fmt == 'fastq':
+        fmt_to_provide_for_write = 'fastq'
+
     reads_pair_dict = {}
-    for read_record in SeqIO.parse(fasta_in, 'fasta'):
+    for read_record in SeqIO.parse(seq_in, input_seq_fmt):
         read_id_base = '.'.join(read_record.id.split('.')[:-1])
         read_strand = read_record.id.split('.')[-1]
         if read_id_base not in reads_pair_dict:
@@ -83,29 +97,19 @@ def sep_paired_and_singleton_reads(fasta_in, fasta_out_r1, fasta_out_r2, fasta_o
         if len(reads_pair_dict[read_base]) == 2:
             read_list_paired.add(read_base)
 
-    fasta_out_r1_handle = open(fasta_out_r1, 'w')
-    fasta_out_r2_handle = open(fasta_out_r2, 'w')
-    fasta_out_singleton_handle = open(fasta_out_singleton, 'w')
-
-    for read_record in SeqIO.parse(fasta_in, 'fasta'):
-
+    fasta_out_r1_handle = open(seq_out_r1, 'w')
+    fasta_out_r2_handle = open(seq_out_r2, 'w')
+    fasta_out_singleton_handle = open(seq_out_singleton, 'w')
+    for read_record in SeqIO.parse(seq_in, input_seq_fmt):
         read_id_base = '.'.join(read_record.id.split('.')[:-1])
         read_strand = read_record.id.split('.')[-1]
-
         if read_id_base in read_list_singleton:
-            fasta_out_singleton_handle.write('>%s\n' % read_record.id)
-            fasta_out_singleton_handle.write('%s\n' % str(read_record.seq))
-
+            SeqIO.write(read_record, fasta_out_singleton_handle, fmt_to_provide_for_write)
         if read_id_base in read_list_paired:
-
             if read_strand == '1':
-                fasta_out_r1_handle.write('>%s\n' % read_record.id)
-                fasta_out_r1_handle.write('%s\n' % str(read_record.seq))
-
+                SeqIO.write(read_record, fasta_out_r1_handle, fmt_to_provide_for_write)
             if read_strand == '2':
-                fasta_out_r2_handle.write('>%s\n' % read_record.id)
-                fasta_out_r2_handle.write('%s\n' % str(read_record.seq))
-
+                SeqIO.write(read_record, fasta_out_r2_handle, fmt_to_provide_for_write)
     fasta_out_r1_handle.close()
     fasta_out_r2_handle.close()
     fasta_out_singleton_handle.close()
@@ -121,6 +125,9 @@ def single_line_seq(fasta_in, fasta_out):
 
 def subsample_paired_and_singleton_reads(paired_r1, paired_r2, singleton_in, subsample_pct, paired_r1_out, paired_r2_out, singleton_out, usearch_exe):
 
+    # check input file format
+    input_seq_ext = os.path.splitext(paired_r1)[1]
+
     # define tmp files
     paired_r1_out_tmp = '%s.tmp' % paired_r1_out
     paired_r2_out_tmp = '%s.tmp' % paired_r2_out
@@ -130,21 +137,27 @@ def subsample_paired_and_singleton_reads(paired_r1, paired_r2, singleton_in, sub
     subsample_cmd_paired_r1 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r1, paired_r1_out_tmp, subsample_pct)
     subsample_cmd_paired_r2 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r2, paired_r2_out_tmp, subsample_pct)
     subsample_cmd_singleton = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, singleton_in, singleton_out_tmp, subsample_pct)
+    if ('q' in input_seq_ext) or ('Q' in input_seq_ext):
+        subsample_cmd_paired_r1 = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r1, paired_r1_out, subsample_pct)
+        subsample_cmd_paired_r2 = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r2, paired_r2_out, subsample_pct)
+        subsample_cmd_singleton = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, singleton_in, singleton_out, subsample_pct)
 
     # execute commands
     os.system(subsample_cmd_paired_r1)
     os.system(subsample_cmd_paired_r2)
     os.system(subsample_cmd_singleton)
 
-    # single line sequences
-    single_line_seq(paired_r1_out_tmp, paired_r1_out)
-    single_line_seq(paired_r2_out_tmp, paired_r2_out)
-    single_line_seq(singleton_out_tmp, singleton_out)
+    if ('q' not in input_seq_ext) and ('Q' not in input_seq_ext):
 
-    # remove tmp files
-    os.remove(paired_r1_out_tmp)
-    os.remove(paired_r2_out_tmp)
-    os.remove(singleton_out_tmp)
+        # single line sequences
+        single_line_seq(paired_r1_out_tmp, paired_r1_out)
+        single_line_seq(paired_r2_out_tmp, paired_r2_out)
+        single_line_seq(singleton_out_tmp, singleton_out)
+
+        # remove tmp files
+        os.remove(paired_r1_out_tmp)
+        os.remove(paired_r2_out_tmp)
+        os.remove(singleton_out_tmp)
 
 
 def subsample_sortmerna_output(sortmerna_op, subsample_pct, sortmerna_op_subsampled, usearch_exe, seqtk_exe):
@@ -233,6 +246,7 @@ def matam_16s(args):
     matam_ref                       = args['d']
     uclust_iden_cutoff              = args['i']
     num_threads                     = args['t']
+    mem_mb                          = args['mem']
     force_overwrite                 = args['force']
     matam_assembly_script           = args['matam']
     usearch_exe                     = args['usearch']
@@ -271,19 +285,26 @@ def matam_16s(args):
             print('%s not found, program exited!' % os.path.basename(reads_file_r2))
             exit()
     else:
-        print('please either provide paired reads (-r1 and -r2) or 16S reads extracted from the paired reads (-r16s), program exited!')
+        print('please either provide paired-end reads (-r1 and -r2) or 16S reads (-r16s) extracted from the paired-end reads as input, program exited!')
         exit()
 
+    input_reads_fmt = ''
+    if extract_16s_reads is True:
+        input_reads_fmt = os.path.splitext(input_16S_reads)
+    else:
+        input_reads_fmt = os.path.splitext(reads_file_r1)
+
+
     ####################################################################################################################
 
-    matam16s_wd                             = '%s_Matam16S_wd'                         % output_prefix
-    log_file                                = '%s/%s_matam_16s.log'                    % (matam16s_wd, output_prefix)
-    combined_r1_r2                          = '%s/%s_combined_R1_R2.fasta'             % (matam16s_wd, output_prefix)
-    extracted_16S_reads                     = '%s/%s_16S_reads.fasta'                  % (matam16s_wd, output_prefix)
-    combined_all_depth_matam_assemblies     = '%s/%s_assembled_16S_unclustered.fasta'  % (matam16s_wd, output_prefix)
-    uclust_output_uc                        = '%s/%s_assembled_16S_uclust_%s.uc'       % (matam16s_wd, output_prefix, uclust_iden_cutoff)
-    uclust_output_txt                       = '%s/%s_assembled_16S_uclust_%s.txt'      % (matam16s_wd, output_prefix, uclust_iden_cutoff)
-    uclust_output_fasta                     = '%s/%s_assembled_16S_uclust_%s.fasta'    % (matam16s_wd, output_prefix, uclust_iden_cutoff)
+    matam16s_wd                             = '%s_Matam16S_wd'                          % output_prefix
+    log_file                                = '%s/%s_matam_16s.log'                     % (matam16s_wd, output_prefix)
+    combined_r1_r2                          = '%s/%s_combined_R1_R2%s'                  % (matam16s_wd, output_prefix, input_reads_fmt)
+    extracted_16S_reads                     = '%s/%s_16S_reads%s'                       % (matam16s_wd, output_prefix, input_reads_fmt)
+    combined_all_depth_matam_assemblies     = '%s/%s_assembled_16S_unclustered.fasta'   % (matam16s_wd, output_prefix)
+    uclust_output_uc                        = '%s/%s_assembled_16S_uclust_%s.uc'        % (matam16s_wd, output_prefix, uclust_iden_cutoff)
+    uclust_output_txt                       = '%s/%s_assembled_16S_uclust_%s.txt'       % (matam16s_wd, output_prefix, uclust_iden_cutoff)
+    uclust_output_fasta                     = '%s/%s_assembled_16S_uclust_%s.fasta'     % (matam16s_wd, output_prefix, uclust_iden_cutoff)
 
     # create folder
     if (os.path.isdir(matam16s_wd) is True) and (force_overwrite is False):
@@ -293,25 +314,30 @@ def matam_16s(args):
         os.mkdir(matam16s_wd)
 
     if extract_16s_reads is True:
+
         # combine R1 and R2
-        report_and_log(('Combining the forward and reverse reads'), log_file, keep_quiet)
+        report_and_log(('Combining R1 and R2 with seqtk'), log_file, keep_quiet)
         combined_r1_r2_cmd = '%s mergepe %s %s > %s' % (seqtk_exe, reads_file_r1, reads_file_r2, combined_r1_r2)
+        report_and_log(combined_r1_r2_cmd, log_file, keep_quiet)
         os.system(combined_r1_r2_cmd)
 
         # Extract 16S reads
         report_and_log(('Extracting 16S reads with Matam'), log_file, keep_quiet)
-        matam_filter_cmd = '%s -i %s -o %s/%s_get_16S_reads_wd --cpu %s --max_memory 100000 -v --filter_only -d %s' % (matam_assembly_script, combined_r1_r2, matam16s_wd, output_prefix, num_threads, matam_ref)
-        report_and_log(matam_filter_cmd, log_file, True)
+        matam_filter_cmd = '%s -i %s -o %s/%s_get_16S_reads_wd --cpu %s --max_memory %s -v --filter_only -d %s' % (matam_assembly_script, combined_r1_r2, matam16s_wd, output_prefix, num_threads, mem_mb, matam_ref)
+        report_and_log(matam_filter_cmd, log_file, keep_quiet)
         os.system(matam_filter_cmd)
 
         # get file name of extracted 16S reads
         matam_16S_reads_re = '%s/%s_get_16S_reads_wd/workdir/*.fasta' % (matam16s_wd, output_prefix)
-        matam_16S_reads    = glob.glob(matam_16S_reads_re)[0]
-        os.system('mv %s %s' % (matam_16S_reads, extracted_16S_reads))
+        matam_16S_reads_list = glob.glob(matam_16S_reads_re)
+        if len(matam_16S_reads_list) == 0:
+            print('16S reads not found, please check if 16S reads extraction step finished successfully.')
+            exit()
 
+        matam_16S_reads    = matam_16S_reads_list[0]
+        os.system('mv %s %s' % (matam_16S_reads, extracted_16S_reads))
     else:
         extracted_16S_reads = input_16S_reads
-        #os.system('cp %s %s' % (input_16S_reads, extracted_16S_reads))
 
 
     # subsample 16S reads and assemble
@@ -320,7 +346,7 @@ def matam_16s(args):
 
         report_and_log(('Subsample RNA reads at %s percent' % subsample_pct), log_file, keep_quiet)
 
-        subsample_reads_file = '%s/%s_16S_reads_subset_%s.fasta'    % (matam16s_wd, output_prefix, subsample_pct)
+        subsample_reads_file = '%s/%s_16S_reads_subset_%s%s'    % (matam16s_wd, output_prefix, subsample_pct, input_reads_fmt)
         matam_output_folder  = '%s/%s_16S_reads_subset_%s_Matam_wd' % (matam16s_wd, output_prefix, subsample_pct)
 
         # subsample
@@ -329,8 +355,8 @@ def matam_16s(args):
         # assemble with Matam
         report_and_log(('Assembling subsampled reads with Matam'), log_file, keep_quiet)
 
-        matam_cmd = '%s -d %s -i %s --cpu %s --max_memory 100000 -v -o %s' % (matam_assembly_script, matam_ref, subsample_reads_file, num_threads, matam_output_folder)
-        report_and_log(matam_cmd, log_file, True)
+        matam_cmd = '%s -d %s -i %s --cpu %s --max_memory %s -v -o %s' % (matam_assembly_script, matam_ref, subsample_reads_file, num_threads, mem_mb, matam_output_folder)
+        report_and_log(matam_cmd, log_file, keep_quiet)
         os.system(matam_cmd)
 
         matam_assemblies          = '%s/workdir/scaffolds.NR.min_500bp.fa'          % (matam_output_folder)
@@ -347,13 +373,13 @@ def matam_16s(args):
     # combine Matam outputs
     report_and_log(('Combine Matam assemblies at all depth'), log_file, keep_quiet)
     combine_cmd = 'cat %s > %s' % (' '.join(renamed_matam_assembly_list), combined_all_depth_matam_assemblies)
-    report_and_log(combine_cmd, log_file, True)
+    report_and_log(combine_cmd, log_file, keep_quiet)
     os.system(combine_cmd)
 
     # dereplicate combined assemblies with Usearch
     report_and_log(('Cluster combined Matam assemblies with UCLUST at %s identity cutoff' % uclust_iden_cutoff), log_file, keep_quiet)
     uclust_cmd = '%s -cluster_fast %s -id %s -centroids %s -uc %s -sort length -quiet' % (usearch_exe, combined_all_depth_matam_assemblies, uclust_iden_cutoff, uclust_output_fasta, uclust_output_uc)
-    report_and_log(uclust_cmd, log_file, True)
+    report_and_log(uclust_cmd, log_file, keep_quiet)
     os.system(uclust_cmd)
 
     # get readable cluster results
@@ -371,20 +397,19 @@ def matam_16s(args):
 if __name__ == '__main__':
 
     matam_16s_parser = argparse.ArgumentParser()
-
     matam_16s_parser.add_argument('-p',              required=True,                                                     help='output prefix')
     matam_16s_parser.add_argument('-r1',             required=False, default=None,                                      help='forward reads')
     matam_16s_parser.add_argument('-r2',             required=False, default=None,                                      help='reverse reads')
-    matam_16s_parser.add_argument('-r16s',           required=False, default=None,                                      help='extracted 16S reads')
+    matam_16s_parser.add_argument('-r16s',           required=False, default=None,                                      help='extracted 16S reads, specify if you already have 16S reads identified from metagenomic dataset')
     matam_16s_parser.add_argument('-pct',            required=True,  type=str, default='1,5,10,25,50,75,100',           help='subsample percentage, must be integer, between 1-100, deafault: 1,5,10,25,50,75,100')
     matam_16s_parser.add_argument('-d',              required=False, type=str,                                          help='MATAM ref db, same as "-d" in Matam')
-    matam_16s_parser.add_argument('-i',              required=False, type=float, default=0.999,                         help='cluster identity cutoff (0-1), default: 0.999')
+    matam_16s_parser.add_argument('-i',              required=False, type=float, default=0.999,                         help='cluster identity cutoff, do not provide value lower than 0.99, default: 0.999')
     matam_16s_parser.add_argument('-t',              required=False, type=int, default=1,                               help='number of threads, default: 1')
+    matam_16s_parser.add_argument('-mem',            required=False, type=int, default=10240,                           help='Maximum memory to be allocated to Matam (in MB), default: 10240')
     matam_16s_parser.add_argument('-force',          required=False, action="store_true",                               help='force overwrite existing results')
     matam_16s_parser.add_argument('-quiet',          required=False, action="store_true",                               help='not report progress')
     matam_16s_parser.add_argument('-matam',          required=False, type=str, default='matam_assembly.py',             help='path to matam_assembly.py, default: matam_assembly.py')
     matam_16s_parser.add_argument('-seqtk',          required=False, type=str, default='seqtk',                         help='path to seqtk executable file, default: seqtk')
     matam_16s_parser.add_argument('-usearch',        required=False, type=str, default='usearch',                       help='path to usearch executable file, default: usearch')
-
     args = vars(matam_16s_parser.parse_args())
     matam_16s(args)
